package com.daedalos.dbcheck;

// ===========================================================================
// IMPORTS
// ===========================================================================
import java.sql.* ;

/**
 * A simple interface to check data in a database.
 * It provides easy to use methods to retrieve single column values
 * or row counts of database tables without fiddling around with
 * JDBC and SQL.
 *
 * @author Manfred Duchrow
 * @version 1.0
 */
public class DatabaseChecker
{
  // =========================================================================
  // CONSTANTS
  // =========================================================================

  // =========================================================================
  // INSTANCE VARIABLES
  // =========================================================================
  private Connection connection = null ;
  // -------------------------------------------------------------------------

  // =========================================================================
  // CLASS METHODS
  // =========================================================================

  // =========================================================================
  // CONSTRUCTORS
  // =========================================================================
  /**
   * Initialize the new instance with connection, that will be used
   * for all database access.
   */
  public DatabaseChecker( Connection conn )
  {
    super() ;
    this.setConnection( conn ) ;
  } // DatabaseChecker()
  // -------------------------------------------------------------------------

  protected ResultSet executeQuery( String sqlCmd )
    throws SQLException
  {
    Statement statement   = null ;

    statement = this.getConnection().createStatement() ;
    this.logSql( sqlCmd ) ;
    return statement.executeQuery( sqlCmd ) ;
  } // executeQuery()
  // -------------------------------------------------------------------------

  protected ResultSet executeQuery( String select, String whereClause )
    throws SQLException
  {
    String sqlCmd   = null ;

    if ( whereClause == null )
    {
      sqlCmd = select ;
    }
    else
    {
      sqlCmd = select + " WHERE " + whereClause ;
    }
    return this.executeQuery( sqlCmd ) ;
  } // executeQuery()
  protected Connection getConnection() { return connection ; }
  // -------------------------------------------------------------------------

  /**
   * Returns the value of the specified column in the first row
   * that matches the given where clause in the specified table .
   * If the column is not of type INTEGER an exception will be thrown.
   * <br>
   * Example:<br>
   * <ul>
   * int value = dbChecker.getIntColumn( "BOOKS", "IN_STOCK", "ISBN='0-261-10357-1'" );
   * </ul>
   * @throws SQLException - if a database access error occurs
   */
  public int getIntColumn( String tableName, String columnName, String whereClause )
    throws SQLException
  {
    ResultSet result    = null ;
    int columnValue  = 0 ;

    result = this.queryColumn( tableName, columnName, whereClause ) ;
    columnValue = result.getInt( columnName ) ;
    result.close();
    return columnValue ;
  } // getIntColumn()
  // -------------------------------------------------------------------------

  /**
   * Returns the value of the specified column in the first row
   * that matches the given where clause in the specified table .
   * If the column is not of type VARCHAR an exception will be thrown.
   * <br>
   * Example:<br>
   * <ul>
   * String value = dbChecker.getStringColumn( "BOOKS", "AUTHOR", "AUTHOR_ID='8729'" );
   * </ul>
   * @throws SQLException - if a database access error occurs
   */
  public String getStringColumn( String tableName, String columnName, String whereClause )
    throws SQLException
  {
    ResultSet result    = null ;
    String columnValue  = null ;

    result = this.queryColumn( tableName, columnName, whereClause ) ;
    columnValue = result.getString( columnName ) ;
    result.close();
    return columnValue ;
  } // getStringColumn()
  // -------------------------------------------------------------------------

  protected void logSql( String sqlCmd )
  {
    // TBD
  } // logSql()
  // -------------------------------------------------------------------------

  // =========================================================================
  // PROTECTED INSTANCE METHODS
  // =========================================================================

  protected ResultSet queryColumn( String tableName, String columnName, String whereClause )
    throws SQLException
  {
    String sqlCmd       = null ;
    ResultSet result    = null ;
    sqlCmd = "SELECT " + columnName + " FROM " + tableName ;
    result = this.executeQuery( sqlCmd, whereClause ) ;
    if ( result.next() )
    	return result ;
    
    result.close() ;
    throw new SQLException( "Row not found for: WHERE " + whereClause ) ;
  } // queryColumn()
  
  // -------------------------------------------------------------------------

  /**
   * Returns the current number of rows in the specified table
   */
  public long rowCount( String tableName )
    throws SQLException
  {
    return this.rowCount( tableName, null ) ;
  } // rowCount()
  // -------------------------------------------------------------------------

  /**
   * Returns the current number of rows in the specified table
   * that match the conditions in the given where clause.
   */
  public long rowCount( String tableName, String whereClause )
    throws SQLException
  {
    String sqlCmd     = null ;
    ResultSet result  = null ;
    long count        = -1 ;

    sqlCmd = "SELECT count(*) FROM " + tableName ;
    result = this.executeQuery( sqlCmd, whereClause ) ;
    if ( result.next() )
      count = result.getLong(1) ;
    result.close();
    return count ;
  } // rowCount()
  // =========================================================================
  // PUBLIC INSTANCE METHODS
  // =========================================================================

  /**
   * Returns whether or not the row that matches the given where clause
   * exists in the specified table.
   * The method returns true, if there is exactly one row that matches.
   * Zero or more than one row returns false.
   */
  public boolean rowExists( String tableName, String whereClause )
    throws SQLException
  {
    return ( this.rowCount( tableName, whereClause ) == 1L ) ;
  } // rowExists()
  // -------------------------------------------------------------------------

  /**
   * Returns whether or not one or more rows exist that match the given
   * where clause in the specified table.
   * 
   * @deprecated due to naming issues. Use 
   * multipleRowsExist(String tableName, String whereClause) 
   * instead.
   */
  public boolean rowsExist( String tableName, String whereClause )
    throws SQLException
  {
    return this.multipleRowsExist(tableName, whereClause) ;
  } // rowsExist()
  // -------------------------------------------------------------------------

  /**
   * Returns whether or not one or more rows exist that match the given
   * where clause in the specified table.
   * The method returns true, if there is any row that matches.
   * Zero rows returns false.
   */
  public boolean multipleRowsExist( String tableName, String whereClause )
    throws SQLException
  {
    return ( this.rowCount( tableName, whereClause ) > 0 ) ;
  } // rowsExist()
  protected void setConnection( Connection newValue ) { connection = newValue ; }
  // -------------------------------------------------------------------------

  /**
   * Returns whether or not the specified table exists in the database
   */
  public boolean tableExists( String tableName )
  {
    boolean exists = true ;

    try
    {
      this.rowCount( tableName ) ;
    }
    catch ( SQLException ex )
    {
      exists = false ;
    }
    return exists ;
  } // tableExists()
} // class DatabaseChecker
