/*********************************************************
 * JUnit Extensions
 * Version 1.5.0
 *
 *
 * (C)opyright 2001-2003 Daedalos Consulting GmbH, Witten
 *
 * Author: Jens Uwe Pipka, Daedalos Consulting GmbH
 *      mailto:jens-uwe.pipka@daedalos.com
 *      http://www.daedalos.com/DE/djux
 *
 *********************************************************/
package com.daedalos.junit;

import java.io.*;
import java.util.*;
import junit.runner.TestCaseClassLoader;

public class TestResourceFactory implements TestResourceFactoryInterface {
	/** Maps test resource classes to symbolic names */
	private static Map testResourceNameMapping = new HashMap();

	/** Maps symbolic names to test resource classes */
	private static Map testResources = new HashMap();
	
	/** TestResource settings */
	private static boolean globalTestResources;
	private static boolean startTestResources;

	/** TestResource configuration files */
	private final static String testResourcesConfigFileName = "TestResources.properties";
	private final static String testResourcesUserConfigFileName = "TestResourcesPreferences.properties";
	
	private static TestResourceFactory testResourceFactory = new TestResourceFactory();
	
	private TestResourceFactory() {
		super();
		this.loadConfiguration();
	}

	protected void addTestResource(String name, TestResource aTestResource) 
		throws TestResourceException {
		if (testResources.containsKey(name)) {
			new TestResourceException(
				"TestResource with logical name " + name + " already registered!");
		}
		if (testResources.containsValue(aTestResource)) {
			new TestResourceException(
				"TestResource " + aTestResource.getName() + " already registered!");
		}
		testResources.put(name, aTestResource);
	}

	protected void addTestResource(String name, Class aTestResourceClass) 
		throws TestResourceException{
		TestResource aTestResource = this.initializeTestResource(aTestResourceClass);
		if (aTestResource != null) {
			this.addTestResource(name, aTestResource);
			this.addTestResourceNameMapping(name, aTestResourceClass);
			aTestResource.setName(name);
		}
	}

	public void addTestResource(String name, String classname) {
		try {
			if ((name == null) || (name.trim().length() == 0)) {
				name = classname;
			}
			Class aTestResourceClass = Class.forName(classname);
			this.addTestResource(name, aTestResourceClass);			
			TestCaseClassLoader.excludeFromReloading(classname);						
			if (startTestResources) {
				startTestResource(name);
			}
		} catch (ClassNotFoundException e) {
			throw new TestResourceException("Class " + classname + " not found!");
		}
	}

	protected void addTestResourceNameMapping(String name, Class aTestResourceClass) {
		this.getTestResourceNameMapping().put(aTestResourceClass, name);
	}

	public boolean getGlobalTestResources() {
		return globalTestResources;
	}

	public boolean getStartTestResources() {
		return startTestResources;
	}

	public TestResource getTestResource(Class aTestResourceClass)
		throws TestResourceNotAvailableException, TestResourceNotRunningException {
		String aName = (String) testResourceNameMapping.get(aTestResourceClass);
		return this.getTestResource(aName);
	}

	public TestResource getTestResource(String name)
		throws TestResourceNotAvailableException, TestResourceNotRunningException {
		TestResource aTestResource = (TestResource) testResources.get(name);

		if (aTestResource == null) {
			throw new TestResourceNotAvailableException();
		}

		if (aTestResource.isStarted() == false) {
			throw new TestResourceNotRunningException();
		}

		return aTestResource;
	}

	public Map getTestResourceNameMapping() {
		return testResourceNameMapping;
	}

	public Map getTestResources() {
		return testResources;
	}

	public boolean globalTestResourceConfigFileExists() {
		return getLoader().getResource(testResourcesConfigFileName) != null;
	}

	protected TestResource initializeTestResource(Class aTestResourceClass) throws TestResourceException {
		TestResource aTestResource = null;
		try {
			aTestResource = (TestResource) aTestResourceClass.newInstance();
		} catch (InstantiationException e) {
			throw new TestResourceException("Could not instantiate " + aTestResourceClass.getName());
		} catch (ClassCastException e) {
			throw new TestResourceException(aTestResourceClass.getName() + " is not a TestResource");
		} catch (IllegalAccessException e) {
			throw new TestResourceException("Illegal Access for " + aTestResourceClass.getName());
		}
		return aTestResource;
	}

	/**
	 * Finds and loads the user and resource configuration files.
	 */
	public void loadConfiguration() {
		try {
			loadUserConfiguration();
			loadTestResourceConfiguration();
		} catch (IOException e) {
			throw new TestResourceException("Could not open configuration file.");
		}
	}

	/**
	 * Loads the resource configuration file.
	 */
	public void loadTestResourceConfiguration() throws IOException {
		String fileSeparator = System.getProperty("file.separator");		
		String cfgFileName = this.getTestResourceConfigPath() 
			+ fileSeparator + testResourcesConfigFileName;
		File cfgFile = new File(cfgFileName);
		if (cfgFile.exists()) {
			FileInputStream fileInStream = new FileInputStream(cfgFileName);
			try {
				Properties properties = new Properties();
				properties.load(fileInStream);
				Enumeration enum = properties.keys();
				while (enum.hasMoreElements()) {
					String resourceName = (String) enum.nextElement();
					String className = (String) properties.getProperty(resourceName);
					try {
						this.addTestResource(resourceName, className);
					} catch (TestResourceException e) {
						System.err.println("Test resource " + resourceName + " not found!");
					}
				}
			} finally {
				fileInStream.close();
			}
		}
	}

	/**
	 * Loads the user configuration file from the user's home directory
	 * if it exists.
	 */
	private void loadUserConfiguration() throws IOException {
		String cfgPath = System.getProperty("user.home");
		String fileSeparator = System.getProperty("file.separator");
		String cfgFileName = cfgPath + fileSeparator + testResourcesUserConfigFileName;

		File cfgFile = new File(cfgFileName);
		if (cfgFile.exists()) {
			readUserConfigFile(cfgFileName);
		} else {
			globalTestResources = globalTestResourceConfigFileExists();			
		}
	}

	/**
	 * Returns a suitable <code>ClassLoader</code> for locating resources
	 */
	private ClassLoader getLoader() {
		return Thread.currentThread().getContextClassLoader();
	}

	private void readUserConfigFile(String cfgFileName) throws IOException {
		Properties config = new Properties();
		FileInputStream fileInStream = new FileInputStream(cfgFileName);

		try {
			String resourceName;
			String className;

			config.load(fileInStream);

			if (config.containsKey("GlobalTestResources")) {
				globalTestResources = new Boolean(config.getProperty("GlobalTestResources")).booleanValue();
			} else {
				globalTestResources = false;
			}

			if (config.containsKey("AutoStartTestResources")) {
				startTestResources = new Boolean(config.getProperty("AutoStartTestResources")).booleanValue();
			} else {
				startTestResources = false;
			}			
			fileInStream.close();			
		} finally {
			fileInStream.close();
		}
	}

	public void registerNewTestResource(String name, String classname) throws TestResourceException {
		try {
			this.addTestResource(name, classname);
			this.saveTestResourceConfigFile();
		} catch  (IOException e) {
			throw new TestResourceException("Configuration files could not be written");
		}
	}

	public void removeTestResource(String name) throws TestResourceException {
		if (this.getTestResources().remove(name) == null) {
			throw new TestResourceException( 
				"TestResource with logical name " + name + " was not registered!");
		} else {
			String fileSeparator = System.getProperty("file.separator");		
			String cfgFileName = getTestResourceConfigPath() 
				+ fileSeparator + testResourcesConfigFileName;
			File cfgFile = new File(cfgFileName);
			if (cfgFile.exists()) {
				try {
					Properties properties = new Properties();
					
					FileInputStream fileInStream = new FileInputStream(cfgFileName);
					properties.load(fileInStream);
					fileInStream.close();
					
					if (properties.containsKey(name)) {
						properties.remove(name);
					}
					
					FileOutputStream fileOutStream = new FileOutputStream(cfgFileName);					
					properties.store(fileOutStream, "Registered TestResources");					
					fileOutStream.close();																
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
	}

	public void reset() {
		testResourceNameMapping = new HashMap();
		testResources = new HashMap();
	}

	public void setGlobalTestResources(boolean newGlobalTestResources) {
		globalTestResources = newGlobalTestResources;
	}

	public void setStartTestResources(boolean newStartTestResources) {
		startTestResources = newStartTestResources;
	}
	
	public void startAllTestResources()
		throws TestResourceNotAvailableException {
		Set testResources = this.getTestResources().keySet();
		for (Iterator i = testResources.iterator(); i.hasNext();) {
			stopTestResource((String) i.next());
		}
	}

	public boolean startTestResource(String name)
		throws TestResourceNotAvailableException {
		TestResource aTestResource = (TestResource) testResources.get(name);

		if (aTestResource == null) {
			this.registerNewTestResource(name,name);
		}

		if (aTestResource.isStopped()) {
			aTestResource.startTestResource();
		}

		return aTestResource.isStarted();
	}

	public void stopAllTestResources()
		throws TestResourceNotAvailableException {
		Set testResources = this.getTestResources().keySet();
		for (Iterator i = testResources.iterator(); i.hasNext();) {
			stopTestResource((String) i.next());
		}
	}

	public boolean stopTestResource(String name)
		throws TestResourceNotAvailableException {
		TestResource aTestResource = (TestResource) testResources.get(name);

		if (aTestResource == null) {
			throw new TestResourceNotAvailableException();
		}

		if (aTestResource.isStarted()) {
			aTestResource.stopTestResource();
		}

		return aTestResource.isStopped();
	}

	/** 
	 * Save TestResource configuration file.
	 */	
	public void saveTestResourceConfigFile() throws IOException {
		String fileSeparator = System.getProperty("file.separator");
		String cfgFileName = getTestResourceConfigPath() 
			+ fileSeparator 
			+ testResourcesConfigFileName;

		try {
			Properties properties = new Properties();
			
			File cfgFile = new File(cfgFileName);
			if (cfgFile.exists()) {
				FileInputStream fileInStream = new FileInputStream(cfgFileName);
				properties.load(fileInStream);
				fileInStream.close();				
			}
			
			Iterator trNamesIteration = this.getTestResources().keySet().iterator();
			Iterator trIteration = this.getTestResources().values().iterator();
			for (; trIteration.hasNext();) {	
				TestResource aTestResource = (TestResource) trIteration.next();
				String resourceName = (String) trNamesIteration.next();
				String className = aTestResource.getClass().getName();
				if (!properties.containsKey(resourceName)) 
					properties.setProperty(resourceName, className);
			}

			FileOutputStream fileOutStream = new FileOutputStream(cfgFileName);		
			properties.store(fileOutStream, "Registered TestResources");		
			fileOutStream.close();																
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private String getTestResourceConfigPath() {
		if (globalTestResources) {
			return getGlobalTestResourceConfigPath();
		} else {
			return System.getProperty("user.home");
		}
	}

	/**
	 * Find directory where the global TestResource config file is saved.
	 * Search user directory and class path, return the first directoy
	 * where a TestTesource config file is found. If it is not found,
	 * return the user directory.
	 *
	 * Suggested by Manfred Duchrow, implemented by Jens Uwe Pipka.
	 *
	 * Creation date: (23.11.2001 19:27:43)
	 */
	private String getGlobalTestResourceConfigPath() {
		String dir = new String();
		String classpath = System.getProperty("java.class.path");
		String pathSeparator = System.getProperty("path.separator");
		StringTokenizer t = new StringTokenizer(classpath, pathSeparator);
		while (t.hasMoreTokens()) {
			dir = t.nextToken();
			if (globalTestResourceConfigFileExists(dir)) {
				return dir;
			}
		}
		return System.getProperty("user.home");
	}	

	private boolean globalTestResourceConfigFileExists(String path) {
		String fileSeparator = System.getProperty("file.separator");
		File cfgFile = new File(path + fileSeparator + testResourcesConfigFileName);
		return cfgFile.exists();
	}	

	/** 
	 * Save user preferences.
	 */
	public void saveUserConfigFile() throws IOException {
		FileOutputStream fileOutStream;
		Properties config = new Properties();
		String cfgPath = System.getProperty("user.home");
		String fileSeparator = System.getProperty("file.separator");

		config.setProperty(
			"GlobalTestResources",
			new Boolean(globalTestResources).toString());
		config.setProperty(
			"AutoStartTestResources",
			new Boolean(startTestResources).toString());

		fileOutStream =
			new FileOutputStream(cfgPath + fileSeparator + testResourcesUserConfigFileName);
		config.store(fileOutStream, "TestResource Configuration");
		fileOutStream.close();
	}
	
	/** 
	 * Version information
	 */
	public String printVersion() {
		return "1.5.0 March 2003, Daedalos Consulting, Jens Uwe Pipka";
	}
	
	/*
	 * Get the singleton
	 */
	public static TestResourceFactory current() {
		return testResourceFactory;
	}
}